/* helper.c */

/*
 *  Copyright (c) 1992, Research Systems Inc.  All rights reserved.
 *   Reproduction by any means whatsoever is  prohibited without express
 *   written permission.
 */

/************************************************************************/
/*									*/
/*	User library routines to help users set up variables		*/
/*									*/
/************************************************************************/

#include        <stdio.h>
#include        <ctype.h>
#include        <rpc/rpc.h>
#include        "export.h"
#include	"rpc_idl.h"
#include	<malloc.h>
#include	<memory.h>
#include	<string.h>

extern void v_fill_string(STRING* str, char* s);

/*
 *  v_ensure_ptr
 *	malloc the Variable field of a varinfo_t structure if that field
 *	has not already been malloc'ed
 *	Return success if Variable field is good.
 */
public int v_ensure_vptr(varinfo_t *var)
{
    if( var->Variable == NULL ) {
	if( (var->Variable = (VPTR)malloc( sizeof(VARIABLE ))) == NULL )
	    return 0;
    }
    return 1;
}

/*
 *  v_make_byte
 *	fill in a varinfo_t structure with a byte variable
 */
public int v_make_byte(REGISTER varinfo_t *var, char *name, unsigned int c)
{
    strncpy( var->Name, name, MAXIDLEN );
    var->Name[ MAXIDLEN ] = 0;

    if( v_ensure_vptr( var ) ) {
	var->Variable->type	= TYP_BYTE;
	var->Variable->flags	= 0;
	var->Variable->value.c	= c;
	return 1;
    }
    return 0;
}


/*
 *  v_make_int
 *	fill in a varinfo_t structure with a short (IDL integer) variable
 */
public int v_make_int(REGISTER varinfo_t *var, char *name, int i)
{
    strncpy( var->Name, name, MAXIDLEN );
    var->Name[ MAXIDLEN ] = 0;

    if( v_ensure_vptr( var ) ) {
	var->Variable->type	= TYP_INT;
	var->Variable->flags	= 0;
	var->Variable->value.i	= i;
	return 1;
    }
    return 0;
}


/*
 *  v_make_long
 *	fill in a varinfo_t structure with a long variable
 */
public int v_make_long(REGISTER varinfo_t *var, char *name, LONG l)
{
    strncpy( var->Name, name, MAXIDLEN );
    var->Name[ MAXIDLEN ] = 0;

    if( v_ensure_vptr( var ) ) {
	var->Variable->type	= TYP_LONG;
	var->Variable->flags	= 0;
	var->Variable->value.l	= l;
	return 1;
    }
    return 0;
}


/*
 *  v_make_float
 *	fill in a varinfo_t structure with a (4 byte)float variable
 */
public int v_make_float(REGISTER varinfo_t *var, char *name, double f)
{
    strncpy( var->Name, name, MAXIDLEN );
    var->Name[ MAXIDLEN ] = 0;

    if( v_ensure_vptr( var ) ) {
	var->Variable->type	= TYP_FLOAT;
	var->Variable->flags	= 0;
	var->Variable->value.f	= f;
	return 1;
    }
    return 0;
}

/*
 *  v_make_double
 *	fill in a varinfo_t structure with a (8 byte) float variable
 */
public int v_make_double(REGISTER varinfo_t *var, char *name, double d)
{
    strncpy( var->Name, name, MAXIDLEN );
    var->Name[ MAXIDLEN ] = 0;

    if( v_ensure_vptr( var ) ) {
	var->Variable->type	= TYP_DOUBLE;
	var->Variable->flags	= 0;
	var->Variable->value.d	= d;
	return 1;
    }
    return 0;
}

/*
 *  v_make_complex
 *	fill in a varinfo_t structure with a COMPLEX type variable.
 *	User passes in the real and imaginary components
 */
public int v_make_complex(REGISTER varinfo_t *var, char *name,
			  double r, double i)
{
    strncpy( var->Name, name, MAXIDLEN );
    var->Name[ MAXIDLEN ] = 0;

    if( v_ensure_vptr( var ) ) {
	var->Variable->type		= TYP_COMPLEX;
	var->Variable->flags		= 0;
	var->Variable->value.cmp.r	= r;
	var->Variable->value.cmp.i	= i;
	return 1;
    }
    return 0;
}

/*
 *  v_make_string
 *	fill in a varinfo_t structure with a string.
 *	This routine will NOT allocate space for the string but use
 *	the pointer given to it (s). This is only a problem if you don't
 *	call set_idl_variable() before freeing s or overwriting the
 *	string.
 */
int v_make_string(REGISTER varinfo_t *var, char *name, char *s)
{
    strncpy( var->Name, name, MAXIDLEN );
    var->Name[ MAXIDLEN ] = 0;

    if( v_ensure_vptr( var ) ) {
	var->Variable->type		= TYP_STRING;
	var->Variable->flags		= 0;
	v_fill_string( &var->Variable->value.str, s );
	return 1;
    }
    return 0;
}

/*
 *  v_fill_string
 *	fill a STRING structure as appropriate (see below regarding NULL)
 */
void v_fill_string(STRING *str, char *s)
{
    str->stype	= 0;

    if( s == NULL ) {
	str->slen	= 0;
	str->s		= NULL;
    } else {
	str->slen	= strlen(s);
	str->s		= s;
    }
}

/*
 *  v_fill_array
 *	fill a varinfo_t structure with an array. Data is not copied, we
 *	just point to it. C.f. v_make_string() regarding possible problems.
 *	Returns TRUE if we filled structure successfully.
 *
 *	Type should be one of TYP_BYTE, TYP_LONG, etc.
 *
 *	NOTE: we always malloc the array structure. If this would
 *	crush valid info in var->Variable->value.arr one should free
 *	it BEFORE calling this function.
 */
int v_fill_array(varinfo_t *var, char* name, int type,
		 int ndims, LONG dims[], UCHAR *data, LONG len)
{
    ARRAY		*arr;
    u_int		n_elements	= 1;
    int			dim;
    int			elem_size;
    int			idl_type;

    if( ndims < 0)
	return 0;
    
    strncpy( var->Name, name, MAXIDLEN );
    var->Name[ MAXIDLEN ] = 0;

    if( !v_ensure_vptr( var ) ) {
	return 0;
    }

    var->Variable->flags	= V_ARR | V_DYNAMIC;
    var->Variable->type		= type;

    switch( type ) {

      case TYP_BYTE:	elem_size	= sizeof( char );	break;
      case TYP_INT:	elem_size	= sizeof( short );	break;
      case TYP_LONG:	elem_size	= sizeof( LONG );	break;
      case TYP_FLOAT:	elem_size	= sizeof( float );	break;
      case TYP_DOUBLE:	elem_size	= sizeof( double );	break;
      case TYP_COMPLEX:	elem_size	= sizeof( COMPLEX );	break;
      case TYP_STRING:	elem_size	= sizeof( STRING );	break;
      default:		return 0;
    }

    if( (arr = (ARRAY*)malloc( sizeof(ARRAY) )) == NULL ) {
	return 0;
    }
    var->Variable->value.arr	= arr;


    /*	compute the number of elements					*/
    
    for( dim = 0; dim < ndims; ++dim ) {
	if( dims[ dim ] <= 0 )
	    return 0;
	n_elements	*= dims[ dim ];
    }

    /*	fill in the array structure information				*/
    arr->flags		= 0;			/* unused		*/
    arr->file_unit	= 0;			/* unused		*/

    arr->elt_len	= elem_size;		/* size of each element	*/
    arr->n_elts		= n_elements;		/* number of elements	*/
    arr->arr_len	= elem_size * n_elements;	/* sizeof(data)	*/

    arr->n_dim		= ndims;		/* number of valid dims	*/
    /* array dimensions	*/
    memcpy( (char *)arr->dim, (char *)dims, sizeof(LONG)*ndims );

    /*	If there is no data given, create space				*/

    if( data == NULL ) {
	arr->data	= (UCHAR *)malloc( arr->arr_len );
	if( arr->data == NULL ) {
	    free(arr);
	    return 0;
	}
	var->Length	= 0;
    } else {

	/*	If user provides the data, we:				*/
	/*	1) Need to remember not to free(3) user data.		*/
	/*	2) Assume that the data is just longi enough to		*/
	/*	fit the data as specified.				*/
		
	arr->data	= data;

	if( len ) {
	    if( len < arr->arr_len ) {
		free(arr);
		return 0;
	    }
	    var->Length	= len;
	}
	else {
	    var->Length	= arr->arr_len;
	}
    }

    return 1;
}
